//////////////////////////////////////////////////////////////////////
////                                                              ////
////  Advantech Co., Ltd	FPGA Design                           ////
////                                                              ////
//////////////////////////////////////////////////////////////////////
////  Project	: TI Shannon EVM FPGA                             ////
////  File name	: chips_dsp_spi.v		                          ////
////  Description                                                 ////
////      		: Shannon EVM FPGA Core MODULE                    ////
////  Created Date                                                ////
////			: 2010/12/31									  ////
//////////////////////////////////////////////////////////////////////
////                                                              ////
//// Copyright (C) 2010 Advantech Co., Ltd                        ////
////                                                              ////
//////////////////////////////////////////////////////////////////////
module chips_dsp_spi (
    rst_n, clk,
	sync_start, 
	sync_r_wn,
	sync_addr,
	sync_data_w,
	reg_data_r,
	rdy,
	DSP_CSB,
	DSP_CLK,
	DSP_MOSI,
	DSP_MISO
);

input rst_n;
input clk;
input sync_start;
input sync_r_wn;
input [6:0] sync_addr;
input [7:0] sync_data_w;

output [7:0] reg_data_r;
output rdy;

output DSP_CSB;
output DSP_CLK;
output DSP_MOSI;
input  DSP_MISO;


reg DSP_CSB;
reg DSP_CLK;
reg DSP_MOSI;

reg	rdy;
reg [7:0] reg_data_r;


reg   s1_start, s2_start, s3_start, s4_start, s5_start, s6_start;

reg		[5:0]	SPI_ST;		
parameter IDLE	=6'h0; 
parameter START	=6'h1; 
parameter SCS1	=6'h2; 
parameter SCS2	=6'h3; 
parameter RW1	=6'h4; 
parameter RW2	=6'h5; 
parameter A61   =6'h6; 
parameter A62	=6'h7;
parameter A51	=6'h8; 
parameter A52	=6'h9; 
parameter A41	=6'ha;
parameter A42	=6'hb;
parameter A31   =6'hc; 
parameter A32	=6'hd;
parameter A21	=6'he; 
parameter A22	=6'hf; 
parameter A11	=6'h10;
parameter A12	=6'h11; 
parameter A01	=6'h12;
parameter A02	=6'h13;
parameter WD71  =6'h14; 
parameter WD72  =6'h15; 
parameter WD61  =6'h17; 
parameter WD62	=6'h18;
parameter WD51	=6'h19; 
parameter WD52	=6'h1a; 
parameter WD41	=6'h1b;
parameter WD42	=6'h1c;
parameter WD31  =6'h1d; 
parameter WD32	=6'h1e;
parameter WD21	=6'h1f; 
parameter WD22	=6'h20; 
parameter WD11	=6'h21;
parameter WD12	=6'h22; 
parameter WD01	=6'h23;
parameter WD02	=6'h24;
parameter RD71  =6'h25; 
parameter RD72  =6'h26; 
parameter RD61  =6'h27; 
parameter RD62	=6'h28;
parameter RD51	=6'h29; 
parameter RD52	=6'h2a; 
parameter RD41	=6'h2b;
parameter RD42	=6'h2c;
parameter RD31  =6'h2d; 
parameter RD32	=6'h2e;
parameter RD21	=6'h2f; 
parameter RD22	=6'h30; 
parameter RD11	=6'h31;
parameter RD12	=6'h32; 
parameter RD01	=6'h33;
parameter RD02	=6'h34;
parameter PCS1	=6'h35; 
parameter PCS2	=6'h36; 
parameter STOP	=6'h37;


always@(posedge clk or negedge rst_n)		
begin
	if (~rst_n) begin
	    s1_start <= 1'b0;
		s2_start <= 1'b0;
		s3_start <= 1'b0;
		s4_start <= 1'b0;
		s5_start <= 1'b0;
		s6_start <= 1'b0;
	end
	else begin		
	    s1_start <= sync_start;
		s2_start <= s1_start;
		s3_start <= s2_start;
		s4_start <= s3_start;
		s5_start <= s4_start;
		s6_start <= s5_start;
	end
end

always@(posedge clk or negedge rst_n)
begin	
      if (~rst_n) begin
	    SPI_ST <= IDLE ;
	    reg_data_r <= 8'h0;
		rdy <= 1'b1;
		DSP_CSB <= 1'b1;
		DSP_CLK <= 1'b0;
		DSP_MOSI <= 1'b0;
	  end
      else begin	  
		case (SPI_ST)
		IDLE: begin
            rdy <= 1'b1;
			DSP_CSB <= 1'b1;
			DSP_CLK <= 1'b0;
			DSP_MOSI <= 1'b0;    
            if ((~s6_start && s3_start) || (s6_start && ~s3_start))			
		        SPI_ST <= START ;
			else
                SPI_ST <= IDLE ;
		end
        START : begin
            rdy <= 1'b0;
			DSP_CSB <= 1'b0;
			DSP_CLK <= 1'b0;
			DSP_MOSI <= 1'b0;     			
		    SPI_ST <= SCS1 ;
		end		
		SCS1 : begin
            rdy <= 1'b0;
			DSP_CSB <= 1'b0;
			DSP_CLK <= 1'b0;
			DSP_MOSI <= 1'b0;     			
		    SPI_ST <= SCS2 ;
		end		
		SCS2 : begin
            rdy <= 1'b0;
			DSP_CSB <= 1'b0;
			DSP_CLK <= 1'b1;
			if (sync_r_wn == 1'b1) 
				DSP_MOSI <= 1'b1;
            else     	
                DSP_MOSI <= 1'b0;	     			
		    SPI_ST <= RW1 ;
		end		
		////////////////////// RW command ///////////////////////
		RW1 : begin
            rdy <= 1'b0;
			DSP_CSB <= 1'b0;
			DSP_CLK <= 1'b0;		
		    SPI_ST <= RW2 ;
		end		
		RW2 : begin
            rdy <= 1'b0;
			DSP_CSB <= 1'b0;
			DSP_CLK <= 1'b1;
			DSP_MOSI <= sync_addr[6];			
		    SPI_ST <= A61 ;
		end		
		/////////////////////////  Address ///////////////////////
		A61 : begin
            rdy <= 1'b0;
			DSP_CSB <= 1'b0;
			DSP_CLK <= 1'b0;	
		    SPI_ST <= A62 ;
		end		
		A62 : begin
            rdy <= 1'b0;
			DSP_CSB <= 1'b0;
			DSP_CLK <= 1'b1;
			DSP_MOSI <= sync_addr[5];		
		    SPI_ST <= A51 ;
		end		
		A51 : begin
            rdy <= 1'b0;
			DSP_CSB <= 1'b0;
			DSP_CLK <= 1'b0;	
		    SPI_ST <= A52 ;
		end		
		A52 : begin
            rdy <= 1'b0;
			DSP_CSB <= 1'b0;
			DSP_CLK <= 1'b1;
			DSP_MOSI <= sync_addr[4];		
		    SPI_ST <= A41 ;
		end		
		A41 : begin
            rdy <= 1'b0;
			DSP_CSB <= 1'b0;
			DSP_CLK <= 1'b0;	
		    SPI_ST <= A42 ;
		end		
		A42 : begin
            rdy <= 1'b0;
			DSP_CSB <= 1'b0;
			DSP_CLK <= 1'b1;
			DSP_MOSI <= sync_addr[3];		
		    SPI_ST <= A31 ;
		end		
		A31 : begin
            rdy <= 1'b0;
			DSP_CSB <= 1'b0;
			DSP_CLK <= 1'b0;	
		    SPI_ST <= A32 ;
		end		
		A32 : begin
            rdy <= 1'b0;
			DSP_CSB <= 1'b0;
			DSP_CLK <= 1'b1;
			DSP_MOSI <= sync_addr[2];		
		    SPI_ST <= A21 ;
		end		
		A21 : begin
            rdy <= 1'b0;
			DSP_CSB <= 1'b0;
			DSP_CLK <= 1'b0;	
		    SPI_ST <= A22 ;
		end		
		A22 : begin
            rdy <= 1'b0;
			DSP_CSB <= 1'b0;
			DSP_CLK <= 1'b1;
			DSP_MOSI <= sync_addr[1];		
		    SPI_ST <= A11 ;
		end		
		A11 : begin
            rdy <= 1'b0;
			DSP_CSB <= 1'b0;
			DSP_CLK <= 1'b0;	
		    SPI_ST <= A12 ;
		end		
		A12 : begin
            rdy <= 1'b0;
			DSP_CSB <= 1'b0;
			DSP_CLK <= 1'b1;
			DSP_MOSI <= sync_addr[0];		
		    SPI_ST <= A01 ;
		end		
		A01 : begin
            rdy <= 1'b0;
			DSP_CSB <= 1'b0;
			DSP_CLK <= 1'b0;	
		    SPI_ST <= A02 ;
		end		
		A02 : begin
            rdy <= 1'b0;
			DSP_CSB <= 1'b0;
			DSP_CLK <= 1'b1;
			DSP_MOSI <= sync_data_w[7];		
		    if (sync_r_wn == 1'b1) 
				SPI_ST <= RD71;
            else     	
                SPI_ST <= WD71;	
		end		
		///////////////////////////// Write Data //////////////////
		WD71 : begin
            rdy <= 1'b0;
			DSP_CSB <= 1'b0;
			DSP_CLK <= 1'b0;
		    SPI_ST <= WD72 ;
		end		
		WD72 : begin
            rdy <= 1'b0;
			DSP_CSB <= 1'b0;
			DSP_CLK <= 1'b1;
			DSP_MOSI <= sync_data_w[6];		
		    SPI_ST <= WD61 ;
		end		
		WD61 : begin
            rdy <= 1'b0;
			DSP_CSB <= 1'b0;
			DSP_CLK <= 1'b0;	
		    SPI_ST <= WD62 ;
		end		
		WD62 : begin
            rdy <= 1'b0;
			DSP_CSB <= 1'b0;
			DSP_CLK <= 1'b1;
			DSP_MOSI <= sync_data_w[5];		
		    SPI_ST <= WD51 ;
		end		
		WD51 : begin
            rdy <= 1'b0;
			DSP_CSB <= 1'b0;
			DSP_CLK <= 1'b0;	
		    SPI_ST <= WD52 ;
		end		
		WD52 : begin
            rdy <= 1'b0;
			DSP_CSB <= 1'b0;
			DSP_CLK <= 1'b1;
			DSP_MOSI <= sync_data_w[4];		
		    SPI_ST <= WD41 ;
		end		
		WD41 : begin
            rdy <= 1'b0;
			DSP_CSB <= 1'b0;
			DSP_CLK <= 1'b0;	
		    SPI_ST <= WD42 ;
		end		
		WD42 : begin
            rdy <= 1'b0;
			DSP_CSB <= 1'b0;
			DSP_CLK <= 1'b1;
			DSP_MOSI <= sync_data_w[3];		
		    SPI_ST <= WD31 ;
		end		
		WD31 : begin
            rdy <= 1'b0;
			DSP_CSB <= 1'b0;
			DSP_CLK <= 1'b0;	
		    SPI_ST <= WD32 ;
		end		
		WD32 : begin
            rdy <= 1'b0;
			DSP_CSB <= 1'b0;
			DSP_CLK <= 1'b1;
			DSP_MOSI <= sync_data_w[2];		
		    SPI_ST <= WD21 ;
		end		
		WD21 : begin
            rdy <= 1'b0;
			DSP_CSB <= 1'b0;
			DSP_CLK <= 1'b0;	
		    SPI_ST <= WD22 ;
		end		
		WD22 : begin
            rdy <= 1'b0;
			DSP_CSB <= 1'b0;
			DSP_CLK <= 1'b1;
			DSP_MOSI <= sync_data_w[1];		
		    SPI_ST <= WD11 ;
		end		
		WD11 : begin
            rdy <= 1'b0;
			DSP_CSB <= 1'b0;
			DSP_CLK <= 1'b0;
		    SPI_ST <= WD12 ;
		end		
		WD12 : begin
            rdy <= 1'b0;
			DSP_CSB <= 1'b0;
			DSP_CLK <= 1'b1;
			DSP_MOSI <= sync_data_w[0];		
		    SPI_ST <= WD01 ;
		end		
		WD01 : begin
            rdy <= 1'b0;
			DSP_CSB <= 1'b0;
			DSP_CLK <= 1'b0;
		    SPI_ST <= WD02 ;
		end		
		WD02 : begin
            rdy <= 1'b0;
			DSP_CSB <= 1'b0;
			DSP_CLK <= 1'b0;
			DSP_MOSI <= sync_data_w[0];		
			SPI_ST <= PCS1;
		end		
		///////////////////////////// Read Data //////////////////
		RD71 : begin
            rdy <= 1'b0;
			DSP_CSB <= 1'b0;
			DSP_CLK <= 1'b0;
			DSP_MOSI <= 1'b0;	
            reg_data_r[7] <= DSP_MISO;				
		    SPI_ST <= RD72 ;
		end		
		RD72 : begin
            rdy <= 1'b0;
			DSP_CSB <= 1'b0;
			DSP_CLK <= 1'b1;
		    SPI_ST <= RD61 ;
		end		
		RD61 : begin
            rdy <= 1'b0;
			DSP_CSB <= 1'b0;
			DSP_CLK <= 1'b0;	
			reg_data_r[6] <= DSP_MISO;			
		    SPI_ST <= RD62 ;
		end		
		RD62 : begin
            rdy <= 1'b0;
			DSP_CSB <= 1'b0;
			DSP_CLK <= 1'b1;
		    SPI_ST <= RD51 ;
		end		
		RD51 : begin
            rdy <= 1'b0;
			DSP_CSB <= 1'b0;
			DSP_CLK <= 1'b0;	
			reg_data_r[5] <= DSP_MISO;
		    SPI_ST <= RD52 ;
		end		
		RD52 : begin
            rdy <= 1'b0;
			DSP_CSB <= 1'b0;
			DSP_CLK <= 1'b1;
		    SPI_ST <= RD41 ;
		end		
		RD41 : begin
            rdy <= 1'b0;
			DSP_CSB <= 1'b0;
			DSP_CLK <= 1'b0;
			reg_data_r[4] <= DSP_MISO;
		    SPI_ST <= RD42 ;
		end		
		RD42 : begin
            rdy <= 1'b0;
			DSP_CSB <= 1'b0;
			DSP_CLK <= 1'b1;
		    SPI_ST <= RD31 ;
		end		
		RD31 : begin
            rdy <= 1'b0;
			DSP_CSB <= 1'b0;
			DSP_CLK <= 1'b0;	
            reg_data_r[3] <= DSP_MISO; 			
		    SPI_ST <= RD32 ;
		end		
		RD32 : begin
            rdy <= 1'b0;
			DSP_CSB <= 1'b0;
			DSP_CLK <= 1'b1;
		    SPI_ST <= RD21 ;
		end		
		RD21 : begin
            rdy <= 1'b0;
			DSP_CSB <= 1'b0;
			DSP_CLK <= 1'b0;	
			reg_data_r[2] <= DSP_MISO;	
		    SPI_ST <= RD22 ;
		end		
		RD22 : begin
            rdy <= 1'b0;
			DSP_CSB <= 1'b0;
			DSP_CLK <= 1'b1;
		    SPI_ST <= RD11 ;
		end		
		RD11 : begin
            rdy <= 1'b0;
			DSP_CSB <= 1'b0;
			DSP_CLK <= 1'b0;
			reg_data_r[1] <= DSP_MISO;				
		    SPI_ST <= RD12 ;
		end		
		RD12 : begin
            rdy <= 1'b0;
			DSP_CSB <= 1'b0;
			DSP_CLK <= 1'b1;
		    SPI_ST <= RD01 ;
		end		
		RD01 : begin
            rdy <= 1'b0;
			DSP_CSB <= 1'b0;
			DSP_CLK <= 1'b0;	
			reg_data_r[0] <= DSP_MISO;
		    SPI_ST <= RD02 ;
		end		
		RD02 : begin
            rdy <= 1'b0;
			DSP_CSB <= 1'b0;
			DSP_CLK <= 1'b0;
			SPI_ST <= PCS1;
		end		
		/////////////////// STOP
		PCS1 : begin
            rdy <= 1'b0;
			DSP_CSB <= 1'b0;
			DSP_CLK <= 1'b0;
			DSP_MOSI <= 1'b0;     			
		    SPI_ST <= PCS2 ;
		end		
		PCS2 : begin
            rdy <= 1'b1;
			DSP_CSB <= 1'b1;
			DSP_CLK <= 1'b0;
			DSP_MOSI <= 1'b0;     			
		    SPI_ST <= STOP ;
		end	
        STOP : begin
            rdy <= 1'b1;
			DSP_CSB <= 1'b1;
			DSP_CLK <= 1'b0;
			DSP_MOSI <= 1'b0;     			
		    SPI_ST <= IDLE ;
		end				
		default : begin
		    rdy <= 1'b1;
			DSP_CSB <= 1'b0;
			DSP_CLK <= 1'b0;
			DSP_MOSI <= 1'b0;     			
		    SPI_ST <= IDLE ;
		end	
		endcase		
	  end
end

endmodule 

