//////////////////////////////////////////////////////////////////////
////                                                              ////
////  Advantech Co., Ltd	FPGA Design                           ////
////                                                              ////
//////////////////////////////////////////////////////////////////////
////  Project	: TI Shannon EVM FPGA                             ////
////  File name	: dsp_gpio_spi.v		                          ////
////  Description                                                 ////
////      		: Shannon EVM FPGA Core MODULE                    ////
////  Created Date                                                ////
////			: 2010/12/29									  ////
//////////////////////////////////////////////////////////////////////
////                                                              ////
//// Copyright (C) 2010 Advantech Co., Ltd                        ////
////                                                              ////
//////////////////////////////////////////////////////////////////////
`ifdef sim_set
	`include "sim_params.v"
`else	
	`include "params.v"
`endif	

module dsp_gpio_spi (
	cs,rw,addr,data_w,data_r,rdy,
	alt_fpga_update_lock_en, lock_reset, fpga_ireset_n, alt_miso_outen,
	DSP_SSEL1,DSP_SCK1,DSP_MISO1,DSP_MOSI1
);

input DSP_SSEL1;
input DSP_SCK1;
input DSP_MOSI1;
output DSP_MISO1;

input  fpga_ireset_n;
input  lock_reset;
output alt_fpga_update_lock_en;
wire   alt_fpga_update_lock_en;
output alt_miso_outen;
wire   alt_miso_outen;

reg [4:0] spi_c;

/*******************************/  
/**** Other		   *****/
/*******************************/

output		cs;
output		rw;
output	[7:0]	addr;
output	[7:0]	data_w;
input	[7:0]	data_r;
output		rdy;

wire	cs;

wire    internal_rst;   

reg		rw;            // read = 1,   write = 0
reg	[7:0]	addr;
reg	[7:0]	data_w = 8'h0;
reg		DSP_MISO1;
reg		rdy;

reg	[15:0]	alt_spi_reg;
reg	[7:0]	spi_r;

assign internal_rst = (~fpga_ireset_n || DSP_SSEL1)? 1'b1 : 1'b0;
assign  alt_fpga_update_lock_en = (alt_spi_reg[15:8]==`FPGA_UPDATE_UNLOCK_CODE)? 1'b1 : 1'b0;

//assign alt_miso_outen = ~DSP_SSEL1 & rw;
//assign cs = DSP_SSEL1;
assign alt_miso_outen = ~internal_rst & rw;
assign cs = internal_rst; 

always@(negedge DSP_SCK1 or posedge internal_rst)		
begin
	if (internal_rst) begin
		spi_c <= 5'b0;
		rw <= 1'b0;
		addr <= 8'h0;
//		data_w <= 8'h0;
		rdy <= 1'b0;
	end
	else begin		
		case (spi_c)
		
			5'h0: begin
				spi_c <= spi_c + 1;
				rw <= DSP_MOSI1;	
				addr <= 8'h0;
				data_w <= 8'h0;	
				rdy <= 1'b0;	
			end

			5'h1: begin
				spi_c <= spi_c + 1;
				rw <= rw;
				addr[6] <= DSP_MOSI1;
				data_w <= 8'h0;		
				rdy <= 1'b0;	
			end

			5'h2: begin
				spi_c <= spi_c + 1;
				rw <= rw;
				addr[5] <= DSP_MOSI1;
				data_w <= 8'h0;		
				rdy <= 1'b0;	
			end

			5'h3: begin
				spi_c <= spi_c + 1;
				rw <= rw;
				addr[4] <= DSP_MOSI1;
				data_w <= 8'h0;	
				rdy <= 1'b0;		
			end

			5'h4: begin
				spi_c <= spi_c + 1;
				rw <= rw;
				addr[3] <= DSP_MOSI1;
				data_w <= 8'h0;	
				rdy <= 1'b0;		
			end

			5'h5: begin
				spi_c <= spi_c + 1;
				rw <= rw;
				addr[2] <= DSP_MOSI1;
				data_w <= 8'h0;	
				rdy <= 1'b0;		
			end

			5'h6: begin
				spi_c <= spi_c + 1;
				rw <= rw;
				addr[1] <= DSP_MOSI1;
				data_w <= 8'h0;	
				rdy <= 1'b0;		
			end	

			5'h7: begin
				spi_c <= spi_c + 1;
				rw <= rw;
				addr[0] <= DSP_MOSI1;
				data_w <= 8'h0;	
				if (~rw) rdy <= 1'b0;
				else rdy <= 1'b1;					
			end	

			5'h8: begin
				spi_c <= spi_c + 1;
				rw <= rw;
				addr <= addr;
				if (~rw) data_w[7] <= DSP_MOSI1;				
				rdy <= 1'b0;
			end

			5'h9: begin
				spi_c <= spi_c + 1;
				rw <= rw;
				addr <= addr;
				if (~rw) data_w[6] <= DSP_MOSI1;				
				rdy <= 1'b0;
			end

			5'ha: begin
				spi_c <= spi_c + 1;
				rw <= rw;
				addr <= addr;
				if (~rw) data_w[5] <= DSP_MOSI1;				
				rdy <= 1'b0;
			end

			5'hb: begin
				spi_c <= spi_c + 1;
				rw <= rw;
				addr <= addr;
				if (~rw) data_w[4] <= DSP_MOSI1;				
				rdy <= 1'b0;
			end
			
			5'hc: begin
				spi_c <= spi_c + 1;
				rw <= rw;
				addr <= addr;
				if (~rw) data_w[3] <= DSP_MOSI1;				
				rdy <= 1'b0;
			end		

			5'hd: begin
				spi_c <= spi_c + 1;
				rw <= rw;
				addr <= addr;
				if (~rw) data_w[2] <= DSP_MOSI1;				
				rdy <= 1'b0;
			end	

			5'he: begin
				spi_c <= spi_c + 1;
				rw <= rw;
				addr <= addr;
				if (~rw) data_w[1] <= DSP_MOSI1;				
				rdy <= 1'b0;
			end	

			5'hf: begin
				spi_c <= spi_c + 1;
				rw <= rw;
				addr <= addr;
				if (~rw) begin
					data_w[0] <= DSP_MOSI1;				
					rdy <= 1'b1;
				end
				else begin
					rdy <= 1'b0;
				end			
			end	

			5'h10: begin
				spi_c <= 5'b0;
				rw <= 1'b0;
				addr <= 8'h0;
				data_w <= 8'h0;
				rdy <= 1'b0;		
			end														
		endcase		
	end
end

always@(posedge DSP_SCK1 or posedge internal_rst)		
begin
	if (internal_rst) DSP_MISO1 <= 1'b0;
	else begin		
		case (spi_c)		
			5'h0: DSP_MISO1 <= 1'b0;	
			5'h1: DSP_MISO1 <= 1'b0;
			5'h2: DSP_MISO1 <= 1'b0;
			5'h3: DSP_MISO1 <= 1'b0;
			5'h4: DSP_MISO1 <= 1'b0;
			5'h5: DSP_MISO1 <= 1'b0;
			5'h6: DSP_MISO1 <= 1'b0;
			5'h7: DSP_MISO1 <= 1'b0;
			5'h8: if (rw) DSP_MISO1 <= spi_r[7];
			5'h9: if (rw) DSP_MISO1 <= spi_r[6];
			5'ha: if (rw) DSP_MISO1 <= spi_r[5];
			5'hb: if (rw) DSP_MISO1 <= spi_r[4];
			5'hc: if (rw) DSP_MISO1 <= spi_r[3];
			5'hd: if (rw) DSP_MISO1 <= spi_r[2];
			5'he: if (rw) DSP_MISO1 <= spi_r[1];
			5'hf: if (rw) DSP_MISO1 <= spi_r[0];
			5'h10: begin
				if ((data_w==8'h00) && (~rw)) DSP_MISO1 <= 1'b1;
				else DSP_MISO1 <= 1'b0;																
			end
		endcase		
	end
end

//////////////////////////////////////////////////////////////////////////////////////////
// FPGA Configuration Registers for alternative DPS GPIO SPI interface to FPGA update
/////////////////////////////////////////////////////////////////////////////////////////
always@(fpga_ireset_n or rw or rdy or cs or addr or data_w or alt_fpga_update_lock_en or lock_reset)		
begin
	if (~fpga_ireset_n) begin
		alt_spi_reg[15:0] <= 16'h0000;  //Reset default value //initial test version
	end
	else if (lock_reset) begin
		alt_spi_reg[15:0] <= 16'h0000;
	end	
//configuration register write	
	else if (~rw && rdy && (~cs)) begin		 
		case (addr)
			8'hE: alt_spi_reg[15:8] <= data_w[7:0]; //FPGA updat lock register
		endcase
	end	
	else begin
		alt_spi_reg[0] <= alt_fpga_update_lock_en; //read only for fpga_update_en	
	end
end	

always@(internal_rst or rw or rdy or alt_spi_reg or addr)		
begin
	if (internal_rst) spi_r <= 8'h0;
//configuration register read		
	else if (rw && rdy) begin
		case (addr)
			8'hD: spi_r <= alt_spi_reg[7:0];
            8'hE: spi_r <= alt_spi_reg[15:8];     // FPGA update lock register			
			default: spi_r <= 8'h0;
		endcase
	end
end		

endmodule 

