//////////////////////////////////////////////////////////////////////
////                                                              ////
////  Advantech Co., Ltd	FPGA Design                           ////
////                                                              ////
//////////////////////////////////////////////////////////////////////
////  Project	: TI Shannon EVM FPGA                             ////
////  File name	: pmbus.v		                                  ////
////  Description                                                 ////
////      		: Shannon EVM FPGA Core MODULE                    ////
////  Created Date                                                ////
////			: 2010/12/09									  ////
//////////////////////////////////////////////////////////////////////
////                                                              ////
//// Copyright (C) 2010 Advantech Co., Ltd                        ////
////                                                              ////
//////////////////////////////////////////////////////////////////////
`define half_clk_cnt 250

module pmbus (
	clk_48m, rst_n,
	start_set,
	i2c_clko,i2c_dat_i,i2c_dat_o,i2c_dat_en
);

input		clk_48m;
input		rst_n;
input		start_set;

output		i2c_clko;
input		i2c_dat_i;
output		i2c_dat_o;
output		i2c_dat_en;


reg	[6:0]	i2c_addr;
reg	[7:0]	i2c_offset;
reg	[15:0]	i2c_datai;

reg	[11:0]	clk_i2c_half_c;
reg			clk_i2c_half;			

reg			i2c_clk;
reg			i2c_dat;
wire		i2c_dat_o;
reg			i2c_dat_en;
reg	[15:0]	i2c_datao;	

reg			i2c_sd0;
reg			i2c_sd1;
reg			i2c_sd2;
reg			i2c_start;
	
reg	[3:0]	I2C_SM;
reg	[4:0]	i2c_c;

parameter SM_IDLE	=4'h0; 
parameter SM_START	=4'h1; 
parameter SM_DEVADDR	=4'h2; 
parameter SM_REGADDR	=4'h3; 
parameter SM_WDATA1	=4'h4; 
parameter SM_WDATA2	=4'h5; 
parameter SM_END	=4'h6; 
parameter SM_START1	=4'h7; 
parameter SM_DEVADDR1	=4'h8; 
parameter SM_RDATA1	=4'h9; 
parameter SM_RDATA2	=4'ha; 
parameter SM_END1	=4'hb; 

wire		i2c_busy;

assign 		i2c_dat_o = i2c_dat ;
assign		i2c_busy = (I2C_SM==SM_IDLE) ? 1'b0 : 1'b1;

///////////// Set UCD9222 VOUT_COMMAND ==> 1.0V
always@(posedge clk_48m or negedge rst_n)		
begin
	if (~rst_n) begin
		i2c_addr <= 8'h0;
		i2c_offset <= 8'h0;
		i2c_datai <= 16'h0;
	end
	else begin
		i2c_addr <= 7'h4E;   //78
		i2c_offset <= 8'h21; //23;
		i2c_datai <= 16'h0fff; //1000;
	end
end	

//////////// 48M -> 1M
always@(posedge clk_48m or negedge rst_n)		
begin
	if (~rst_n) begin
	    clk_i2c_half_c <= 12'h0;
		clk_i2c_half <= 1'b0;
	end	
	else if (clk_i2c_half_c >= `half_clk_cnt-1) begin  
		clk_i2c_half_c <= 12'h0;
		clk_i2c_half <= ~clk_i2c_half;
	end
	else begin
		clk_i2c_half_c <= clk_i2c_half_c + 1;
	end	
end


// Async sync for the I2C start        ////////////////////////
always@(posedge clk_i2c_half or negedge rst_n)		
begin
	if (~rst_n) begin
		i2c_sd0 <= 1'b0;
		i2c_sd1 <= 1'b0;
		i2c_sd2 <= 1'b0;
	end	
	else begin 
		i2c_sd0 <= start_set;
		i2c_sd1 <= i2c_sd0;
		i2c_sd2 <= i2c_sd1;
	end	
end

always@(posedge clk_i2c_half or negedge rst_n)		
begin
	if (~rst_n) 
		i2c_start <= 1'b0;
	else if (~i2c_sd2 && i2c_sd1)     // rising
		i2c_start <= 1'b1;
	else 
		i2c_start <= 1'b0;
end

///////////////////// I2C FSM  ////////////////////////////////////
always@(posedge clk_i2c_half or negedge rst_n)		
begin
	if (~rst_n) begin
		i2c_clk <= 1'b1;
		i2c_dat <= 1'b1;
		I2C_SM <= SM_IDLE;
		i2c_c <= 5'h0;
		i2c_dat_en <= 1'b0;
		i2c_datao <= 16'b0;
	end
	else begin   
	    case (I2C_SM)
	    
		SM_IDLE: begin
			if (i2c_start) begin
				i2c_clk <= 1'b1;
				i2c_dat <= 1'b0;			
				I2C_SM <= SM_START;	
				i2c_c <= 5'h0;		
				i2c_dat_en <= 1'b1;	
			end
			else begin
				i2c_clk <= 1'b1;
				i2c_dat <= 1'b1;			
				I2C_SM <= SM_IDLE;
				i2c_c <= 5'h0;	
				i2c_dat_en <= 1'b0;			
			end
		end		

		SM_START: begin
			if (i2c_c > 5'h3) begin
			//if (i2c_c > 5'h6) begin
				i2c_clk <= 1'b0;
				i2c_dat <= i2c_addr[6];  // falling driving
			//	i2c_dat <= 1'b0;			
				I2C_SM <= SM_DEVADDR;
				i2c_c <= 5'h1;     // falling driving
			//	i2c_c <= 5'h0;
				i2c_dat_en <= 1'b1;		
			end
			else begin
				i2c_clk <= 1'b1;  
				i2c_dat <= 1'b0;			
				I2C_SM <= SM_START;
				i2c_c <= i2c_c + 1;
				i2c_dat_en <= 1'b1;				
			end
		end	

		SM_DEVADDR: begin
			if (i2c_c > 5'h17) begin
				i2c_clk <= 1'b0;
				i2c_dat <= i2c_offset[7];
			//	i2c_dat <= 1'b0;			
				I2C_SM <= SM_REGADDR;
				i2c_c <= 5'h1;
			//	i2c_c <= 5'h0;	
				i2c_dat_en <= 1'b1;			
			end					
			else if (i2c_c > 5'h11) begin
				i2c_clk <= 1'b0;
				i2c_dat <= 1'b0;			
				I2C_SM <= SM_DEVADDR;
				i2c_c <= i2c_c + 1;	
				i2c_dat_en <= 1'b1;			
			end
			else begin
				i2c_clk <= ~i2c_clk;
				I2C_SM <= SM_DEVADDR;
				i2c_c <= i2c_c + 1;	
				case (i2c_c)		
					5'h0: begin
						i2c_dat <= i2c_addr[6];
						i2c_dat_en <= 1'b1;
					end
					5'h2: begin
						i2c_dat <= i2c_addr[5];
						i2c_dat_en <= 1'b1;
					end
					5'h4: begin
						i2c_dat <= i2c_addr[4];
						i2c_dat_en <= 1'b1;
					end
					5'h6: begin
						i2c_dat <= i2c_addr[3];
						i2c_dat_en <= 1'b1;
					end
					5'h8: begin
						i2c_dat <= i2c_addr[2];
						i2c_dat_en <= 1'b1;
					end
					5'ha: begin
						i2c_dat <= i2c_addr[1];
						i2c_dat_en <= 1'b1;
					end	
					5'hc: begin
						i2c_dat <= i2c_addr[0];
						i2c_dat_en <= 1'b1;
					end					
					5'he: begin
						i2c_dat <= 1'b0;      // write for default set == vout_command
						i2c_dat_en <= 1'b1;
					end
					5'h10: begin
						i2c_dat <= 1'b0;      // check ack
						i2c_dat_en <= 1'b0;
					end
				endcase			
			end
		end
								
		SM_REGADDR: begin			
			if (i2c_c > 5'h17) begin
				i2c_clk <= 1'b0;
				i2c_dat <= i2c_datai[7];
				//i2c_dat <= 1'b0;			
				if (i2c_sd2) I2C_SM <= SM_WDATA1;
				else I2C_SM <= SM_END;
				i2c_c <= 5'h1;
				//i2c_c <= 5'h0;	
				i2c_dat_en <= 1'b1;			
			end			
			else if (i2c_c > 5'h11) begin
				i2c_clk <= 1'b0;
				i2c_dat <= 1'b0;			
				I2C_SM <= SM_REGADDR;
				i2c_c <= i2c_c + 1;		
				i2c_dat_en <= 1'b1;		
			end
			else begin
				i2c_clk <= ~i2c_clk;
				I2C_SM <= SM_REGADDR;
				i2c_c <= i2c_c + 1;
				case (i2c_c)
					5'h0: begin
						i2c_dat <= i2c_offset[7];
						i2c_dat_en <= 1'b1;
					end			
					5'h2: begin
						i2c_dat <= i2c_offset[6];
						i2c_dat_en <= 1'b1;
					end	
					5'h4: begin
						i2c_dat <= i2c_offset[5];
						i2c_dat_en <= 1'b1;
					end
					5'h6: begin
						i2c_dat <= i2c_offset[4];
						i2c_dat_en <= 1'b1;
					end
					5'h8: begin
						i2c_dat <= i2c_offset[3];
						i2c_dat_en <= 1'b1;
					end
					5'ha: begin
						i2c_dat <= i2c_offset[2];
						i2c_dat_en <= 1'b1;
					end
					5'hc: begin
						i2c_dat <= i2c_offset[1];
						i2c_dat_en <= 1'b1;
					end
					5'he: begin
						i2c_dat <= i2c_offset[0];
						i2c_dat_en <= 1'b1;
					end
					5'hf: begin
						i2c_dat <= i2c_offset[0];
						i2c_dat_en <= 1'b1;	
					end
					5'h10: begin
						i2c_dat <= 1'b0;
						i2c_dat_en <= 1'b0;
					end
				endcase				
			end
		end

		SM_WDATA1: begin			
			if (i2c_c > 5'h17) begin
				i2c_clk <= 1'b0;
				i2c_dat <= i2c_datai[15];
				//i2c_dat <= 1'b0;
				//if (vccpg) I2C_SM <= SM_END;
				//else 
				I2C_SM <= SM_WDATA2;
				i2c_c <= 5'h1;
			//	i2c_c <= 5'h0;	
				i2c_dat_en <= 1'b1;			
			end			
			else if (i2c_c > 5'h11) begin
				i2c_clk <= 1'b0;
				i2c_dat <= 1'b0;			
				I2C_SM <= SM_WDATA1;
				i2c_c <= i2c_c + 1;		
				i2c_dat_en <= 1'b1;		
			end
			else begin
				i2c_clk <= i2c_clk + 1;
				I2C_SM <= SM_WDATA1;
				i2c_c <= i2c_c + 1;	
				case (i2c_c)
					5'h0: begin
						i2c_dat <= i2c_datai[7];
						i2c_dat_en <= 1'b1;
					end			
					5'h2: begin
						i2c_dat <= i2c_datai[6];
						i2c_dat_en <= 1'b1;
					end	
					5'h4: begin
						i2c_dat <= i2c_datai[5];
						i2c_dat_en <= 1'b1;
					end
					5'h6: begin
						i2c_dat <= i2c_datai[4];
						i2c_dat_en <= 1'b1;
					end
					5'h8: begin
						i2c_dat <= i2c_datai[3];
						i2c_dat_en <= 1'b1;
					end
					5'ha: begin
						i2c_dat <= i2c_datai[2];
						i2c_dat_en <= 1'b1;
					end
					5'hc: begin
						i2c_dat <= i2c_datai[1];
						i2c_dat_en <= 1'b1;
					end
					5'he: begin
						i2c_dat <= i2c_datai[0];
						i2c_dat_en <= 1'b1;
					end
					5'h10: begin
						i2c_dat <= 1'b0;
						i2c_dat_en <= 1'b0;
					end
				endcase			
			end
		end

		SM_WDATA2: begin			
			if (i2c_c > 5'h17) begin
				i2c_clk <= 1'b0;
				i2c_dat <= 1'b0;			
				I2C_SM <= SM_END;
				i2c_c <= 5'h0;	
				i2c_dat_en <= 1'b1;			
			end			
			else if (i2c_c > 5'h11) begin
				i2c_clk <= 1'b0;
				i2c_dat <= 1'b0;			
				I2C_SM <= SM_WDATA2;
				i2c_c <= i2c_c + 1;		
				i2c_dat_en <= 1'b1;		
			end
			else begin
				i2c_clk <= i2c_clk + 1;
				I2C_SM <= SM_WDATA2;
				i2c_c <= i2c_c + 1;	
				case (i2c_c)
					5'h0: begin
						i2c_dat <= i2c_datai[15];
						i2c_dat_en <= 1'b1;
					end			
					5'h2: begin
						i2c_dat <= i2c_datai[14];
						i2c_dat_en <= 1'b1;
					end	
					5'h4: begin
						i2c_dat <= i2c_datai[13];
						i2c_dat_en <= 1'b1;
					end
					5'h6: begin
						i2c_dat <= i2c_datai[12];
						i2c_dat_en <= 1'b1;
					end
					5'h8: begin
						i2c_dat <= i2c_datai[11];
						i2c_dat_en <= 1'b1;
					end
					5'ha: begin
						i2c_dat <= i2c_datai[10];
						i2c_dat_en <= 1'b1;
					end
					5'hc: begin
						i2c_dat <= i2c_datai[9];
						i2c_dat_en <= 1'b1;
					end
					5'he: begin
						i2c_dat <= i2c_datai[8];
						i2c_dat_en <= 1'b1;
					end
					5'h10: begin
						i2c_dat <= 1'b0;
						i2c_dat_en <= 1'b0;
					end
				endcase			
			end
		end
		
		SM_END: begin
			case (i2c_c)
				5'h0: begin
					i2c_clk <= 1'b0;
					i2c_dat <= 1'b0;			
					I2C_SM <= SM_END;
					i2c_c <= i2c_c + 1;	
					i2c_dat_en <= 1'b1;		
				end
				5'h1: begin
					i2c_clk <= 1'b1;
					i2c_dat <= 1'b0;			
					I2C_SM <= SM_END;
					i2c_c <= i2c_c + 1;
					i2c_dat_en <= 1'b1;			
				end
				5'h2: begin
					i2c_clk <= 1'b1;
					i2c_dat <= 1'b1;			
					I2C_SM <= SM_END;
					i2c_c <= i2c_c + 1;	
					i2c_dat_en <= 1'b1;			
				end
				5'h3: begin
					i2c_clk <= 1'b1;
					i2c_dat <= 1'b1;			
					I2C_SM <= SM_END;
					i2c_c <= i2c_c + 1;
					i2c_dat_en <= 1'b1;			
				end				
				5'h4: begin
					i2c_clk <= 1'b1;			
					if (~i2c_sd2) begin
						I2C_SM <= SM_START1;
						i2c_dat <= 1'b0;
						i2c_dat_en <= 1'b1;
					end
					else begin
						I2C_SM <= SM_IDLE;
						i2c_dat <= 1'b1;
						i2c_dat_en <= 1'b0;
					end
					i2c_c <= 5'h0;				
				end	
				default: i2c_c <= 5'h0;
			endcase					
		end

		SM_START1: begin
			if (i2c_c > 4'h6) begin
				i2c_clk <= 1'b0;
				i2c_dat <= 1'b0;			
				I2C_SM <= SM_DEVADDR1;
				i2c_c <= 5'h0;	
				i2c_dat_en <= 1'b1;			
			end
			else begin
				i2c_clk <= 1'b0;
				i2c_dat <= 1'b0;			
				I2C_SM <= SM_START1;
				i2c_c <= i2c_c + 1;
				i2c_dat_en <= 1'b1;				
			end
		end	

		SM_DEVADDR1: begin
			if (i2c_c > 5'h17) begin
				i2c_clk <= 1'b0;
				i2c_dat <= 1'b0;
				I2C_SM <= SM_RDATA1;			
				i2c_c <= 5'h0;	
				i2c_dat_en <= 1'b1;			
			end					
			else if (i2c_c > 5'h11) begin
				i2c_clk <= 1'b0;
				i2c_dat <= 1'b0;			
				I2C_SM <= SM_DEVADDR1;
				i2c_c <= i2c_c + 1;	
				i2c_dat_en <= 1'b1;			
			end
			else begin
				i2c_clk <= i2c_clk + 1;
				I2C_SM <= SM_DEVADDR1;
				i2c_c <= i2c_c + 1;
				case (i2c_c)	
					5'h0: begin
						i2c_dat <= i2c_addr[6];
						i2c_dat_en <= 1'b1;
					end
					5'h2: begin
						i2c_dat <= i2c_addr[5];
						i2c_dat_en <= 1'b1;
					end
					5'h4: begin
						i2c_dat <= i2c_addr[4];
						i2c_dat_en <= 1'b1;
					end
					5'h6: begin
						i2c_dat <= i2c_addr[3];
						i2c_dat_en <= 1'b1;
					end
					5'h8: begin
						i2c_dat <= i2c_addr[2];
						i2c_dat_en <= 1'b1;
					end
					5'ha: begin
						i2c_dat <= i2c_addr[1];
						i2c_dat_en <= 1'b1;
					end	
					5'hc: begin
						i2c_dat <= i2c_addr[0];
						i2c_dat_en <= 1'b1;
					end	
					5'he: begin
						i2c_dat <= 1'b1;
						i2c_dat_en <= 1'b1;
					end
					5'h10: begin
						i2c_dat <= 1'b0;
						i2c_dat_en <= 1'b0;
					end
				endcase				
			end
		end

		SM_RDATA1: begin
			if (i2c_c > 5'h17) begin
				i2c_clk <= 1'b0;
				i2c_dat <= 1'b0;
				if (i2c_sd2) I2C_SM <= SM_END1;
				else I2C_SM <= SM_RDATA2;				
				i2c_c <= 5'h0;	
				i2c_dat_en <= 1'b1;			
			end					
			else if (i2c_c > 5'h11) begin
				i2c_clk <= 1'b0;
				i2c_dat <= 1'b0;			
				I2C_SM <= SM_RDATA1;
				i2c_c <= ~i2c_c;	
				i2c_dat_en <= 1'b1;			
			end
			else begin
				i2c_clk <= i2c_clk + 1;
				I2C_SM <= SM_RDATA1;
				i2c_c <= i2c_c + 1;	
				case (i2c_c)	
					5'h0: begin
						i2c_dat_en <= 1'b0;
					end
					5'h1: begin
						i2c_datao[7] <= i2c_dat_i;
						i2c_dat_en <= 1'b0;
					end
					5'h3: begin
						i2c_datao[6] <= i2c_dat_i;
						i2c_dat_en <= 1'b0;
					end
					5'h5: begin
						i2c_datao[5] <= i2c_dat_i;
						i2c_dat_en <= 1'b0;
					end
					5'h7: begin
						i2c_datao[4] <= i2c_dat_i;
						i2c_dat_en <= 1'b0;
					end
					5'h9: begin
						i2c_datao[3] <= i2c_dat_i;
						i2c_dat_en <= 1'b0;
					end
					5'hb: begin
						i2c_datao[2] <= i2c_dat_i;
						i2c_dat_en <= 1'b0;
					end	
					5'hd: begin
						i2c_datao[1] <= i2c_dat_i;
						i2c_dat_en <= 1'b0;
					end	
					5'hf: begin
						i2c_datao[0] <= i2c_dat_i;
						i2c_dat_en <= 1'b0;
					end
					5'h10: begin
						i2c_dat <= 1'b0;
						if (i2c_sd2) i2c_dat_en <= 1'b0;
						else i2c_dat_en <= 1'b1;
					end
				endcase			
			end
		end

		SM_RDATA2: begin
			if (i2c_c > 5'h17) begin
				i2c_clk <= 1'b0;
				i2c_dat <= 1'b0;			
				I2C_SM <= SM_END1;
				i2c_c <= 5'h0;	
				i2c_dat_en <= 1'b1;			
			end					
			else if (i2c_c > 5'h11) begin
				i2c_clk <= 1'b0;
				i2c_dat <= 1'b0;			
				I2C_SM <= SM_RDATA2;
				i2c_c <= i2c_c + 1;	
				i2c_dat_en <= 1'b1;			
			end
			else begin
				i2c_clk <= i2c_clk + 1;
				I2C_SM <= SM_RDATA2;
				i2c_c <= i2c_c + 1;	
				case (i2c_c)	
					5'h0: begin
						i2c_dat_en <= 1'b0;
					end
					5'h1: begin
						i2c_datao[15] <= i2c_dat_i;
						i2c_dat_en <= 1'b0;
					end
					5'h3: begin
						i2c_datao[14] <= i2c_dat_i;
						i2c_dat_en <= 1'b0;
					end
					5'h5: begin
						i2c_datao[13] <= i2c_dat_i;
						i2c_dat_en <= 1'b0;
					end
					5'h7: begin
						i2c_datao[12] <= i2c_dat_i;
						i2c_dat_en <= 1'b0;
					end
					5'h9: begin
						i2c_datao[11] <= i2c_dat_i;
						i2c_dat_en <= 1'b0;
					end
					5'hb: begin
						i2c_datao[10] <= i2c_dat_i;
						i2c_dat_en <= 1'b0;
					end	
					5'hd: begin
						i2c_datao[9] <= i2c_dat_i;
						i2c_dat_en <= 1'b0;
					end	
					5'hf: begin
						i2c_datao[8] <= i2c_dat_i;
						i2c_dat_en <= 1'b0;
					end
					5'h10: begin
						i2c_dat <= 1'b0;
						i2c_dat_en <= 1'b1;
					end
				endcase			
			end
		end
		
		SM_END1: begin
			case (i2c_c)
				5'h0: begin
					i2c_clk <= 1'b0;
					i2c_dat <= 1'b0;			
					I2C_SM <= SM_END1;
					i2c_c <= i2c_c + 1;	
					i2c_dat_en <= 1'b1;		
				end
				5'h1: begin
					i2c_clk <= 1'b1;
					i2c_dat <= 1'b0;			
					I2C_SM <= SM_END1;
					i2c_c <= i2c_c + 1;	
					i2c_dat_en <= 1'b1;		
				end
				5'h2: begin
					i2c_clk <= 1'b1;
					i2c_dat <= 1'b1;			
					I2C_SM <= SM_END1;
					i2c_c <= i2c_c + 1;	
					i2c_dat_en <= 1'b1;			
				end
				5'h3: begin
					i2c_clk <= 1'b1;
					i2c_dat <= 1'b1;			
					I2C_SM <= SM_END1;
					i2c_c <= i2c_c + 1;
					i2c_dat_en <= 1'b1;			
				end				
				5'h4: begin
					i2c_clk <= 1'b1;
					i2c_dat <= 1'b1;			
					I2C_SM <= SM_IDLE;
					i2c_c <= 5'h0;	
					i2c_dat_en <= 1'b0;			
				end	
			default: i2c_c <= 5'h0;
			endcase	
		end
																		
		endcase				
	end
end

wire		i2c_clear;

reg		i2c_err;

always@(posedge clk_i2c_half or negedge rst_n)		
begin
	if (~rst_n) begin
		i2c_err <= 1'b0;
	end
	else begin
		if (i2c_clear) i2c_err <= 1'b0;
		else if ((I2C_SM==SM_DEVADDR) && (i2c_c==5'h11) && i2c_dat_i) i2c_err <= 1'b1;
		else if ((I2C_SM==SM_REGADDR) && (i2c_c==5'h11) && i2c_dat_i) i2c_err <= 1'b1;
		else if ((I2C_SM==SM_WDATA1) && (i2c_c==5'h11) && i2c_dat_i) i2c_err <= 1'b1;
		else if ((I2C_SM==SM_WDATA2) && (i2c_c==5'h11) && i2c_dat_i) i2c_err <= 1'b1;
		else if ((I2C_SM==SM_DEVADDR1) && (i2c_c==5'h11) && i2c_dat_i) i2c_err <= 1'b1;
		//else if ((I2C_SM==SM_RDATA1) && (i2c_c==5'h11) && i2c_dat_i) i2c_err <= 1'b1;
	end
end

reg	[4:0]	CLK_c;

/////////////////20101210 for reset initialization
wire r_i2c_clk;
assign r_i2c_clk = (rst_n==1'b0) ? 1'b0 : i2c_clk;

//always@(posedge clk_48m or negedge i2c_clk)	
always@(posedge clk_48m or negedge r_i2c_clk)	
begin
//	if (~i2c_clk) CLK_c <= 5'h0;
	if (~r_i2c_clk) CLK_c <= 5'h0;
	else if (CLK_c==5'h1f) CLK_c <= 5'h1f;
	else CLK_c <= CLK_c + 1;
end

wire i2c_clko;

assign i2c_clko = (CLK_c==5'h1f) ? i2c_clk : 1'b0;

endmodule 

