//////////////////////////////////////////////////////////////////////
////                                                              ////
////  Advantech Co., Ltd	FPGA Design                           ////
////                                                              ////
//////////////////////////////////////////////////////////////////////
////  Project	: TI Shannon EVM FPGA                             ////
////  File name	: PRNG.v		                                  ////
////  Description                                                 ////
////      		: pseudo-random number generator MODULE           ////
////  Created Date                                                ////
////			: 2010/12/09									  ////
//////////////////////////////////////////////////////////////////////
////                                                              ////
//// Copyright (C) 2010 Advantech Co., Ltd                        ////
////                                                              ////
//////////////////////////////////////////////////////////////////////
`ifdef sim_set
	`include "sim_params.v"
`else	
	`include "params.v"
`endif	

module prng( FPGA_48MHz, FPGA_rst, PRNG_OUTPUT );
input         FPGA_48MHz;
input         FPGA_rst;
output        PRNG_OUTPUT;
/*
//Interlock
//Description:	Slow PRBS (pseudo-random bit sequence) signal that can be sampled by a GPIO on the DM6441 to determine attachment to CI EVM
//Clock:		10KHz +/- 10%
//Voltage:		Signal compatible with LVCMOS 1.8V, can be open drain with pull-up to 1.8V
//PRBS:			9-bit PRNG (pseudo-random number generator) creating 511-bit sequence

*/
reg PRNG_CLK;
reg	[8:0] PRNG;

reg [11:0] clk_count; // for 48MHz to 10KHz = div 4800 == 0x12C0  ==> half clock = 0x960 = 0~12'h95F )

wire PRNG_OUTPUT;

assign PRNG_OUTPUT = PRNG[8];

always@(posedge FPGA_48MHz or negedge FPGA_rst)
  begin
    if(~FPGA_rst)
      begin
        PRNG_CLK <= 1'b0;
		clk_count[11:0] <= 12'h000;
      end
    else if (clk_count[11:0]>= `PRNG_CLK_CNT)
      begin
	    PRNG_CLK <= ~PRNG_CLK;
		clk_count[11:0] <= 12'h000;
	  end	
    else
      begin
         clk_count[11:0] <= clk_count[11:0] + 1;   
      end
  end


always@(posedge PRNG_CLK or negedge FPGA_rst)
  begin
    if(~FPGA_rst)
      begin
        PRNG[8:0] <= 9'h1ff;
      end
    else
      begin
        if (PRNG[8:0] == 9'h000)
          begin
            PRNG[8:0] <= 9'h1ff;
          end
        else
          begin
            PRNG[8:1] <= PRNG[7:0];
            PRNG[0] <= PRNG[0] ^ PRNG[8];
          end
      end
  end

endmodule

